;**********************************************
; MTASK's Include File for Applications v0.0.1
;**********************************************
; BASE KERNEL OFFSET :
kernel=$8000
;
version=$0100
stackl=$40 ; $00-$f0, size of the stack, lower=more, higher=less
           ; (Note: Stack grows backwards)
           ; With larger stacks it takes more time to switch
           ; between tasks, so think carefully do you really
           ; need huge $f0 byte f.e. stacks?
           ; $40 is actually quite nice value.
           ; Even on SCPU this really matters, even if
           ; task switching doesn't take time it anyway decreases
           ; the time that could be used more wisely.

; ** Stacks allocation **
;
; Memory region $7000-$7fff contains storage space for
; 16 processes stacks.
;
; (Note: KERNEL is relocable now and can be relocated f.e. to $2000,
;        which'll mean that stacks will be located at $1000-$1fff)
;
stack1=kernel-$1000  ; Virtual stack1 location in physical memory
stack2=kernel-$0f00  ; --------stack2--------
stack3=kernel-$0e00  ; --------stack3--------
stack4=kernel-$0d00  ; --------stack4--------
stack5=kernel-$0c00  ; --------stack5--------
stack6=kernel-$0b00  ; --------stack6--------
stack7=kernel-$0a00  ; --------stack7--------
stack8=kernel-$0900  ; --------stack8--------
stack9=kernel-$0800  ; --------stack9--------
stacka=kernel-$0700  ; --------stacka--------
stackb=kernel-$0600  ; --------stackb--------
stackc=kernel-$0500  ; --------stackc--------
stackd=kernel-$0400  ; --------stackd--------
stacke=kernel-$0300  ; --------stacke--------
stackf=kernel-$0200  ; --------stackf--------
stack10=kernel-$0100 ; --------stack10--------

; Task mode const's
dead=0
worker=1
idler=2

; SYSTEM VARIABLES (Critical!)
stdin=$9e00  ; STDIN - Stdin buffer
stderr=$9f00 ; STDERR - Errors buffer here
cpar=$03e0   ; C compiler support reserved!
power=$03f0  ; 32bit value that tells how much horse power
             ; MTASK has left to run new threads.
pristd=$03fb ; PRIMARY STD CASING :
             ; If this variable is set then
             ; all STD routines will be executed
             ; in non task switchable mode.
             ; (If multitasking is slowing down one single process,
             ;  then it is possible to set this variable to
             ;  make screen output look smoother atleast).
stdright=$03fc ; Don't set manually!(0=Rights not reserved,
               ; !=thread id that has reserved the std IN/OUT rights.)
curtask=$03fd ; Currently running task
action=$03fe
alts=$03ff ;Allow task switch (0=False, 1=True)
delayv=16 ; This value sets the frequence of task scheduling,
          ; smaller value = quicker
          ; larger value = slower
          ; Quicker means that more CPU power will be reserved for
          ; task switching.(Default 16 is just fine)
          ; Too small values(like 1-8) may jam the processor on
          ; C64's without SCPU or any other speeder.
maxtasks=15 ; Valid values = 1,3,7,15,...
            ; 1=2, 3=4, 7=8, 15=16, .. (acts like AND)

eof=$90

; Kernel Calls
install=        kernel+$00*3
engine=         kernel+$01*3
irq=            kernel+$02*3
load=           kernel+$03*3
save=           kernel+$04*3
calstrlen=      kernel+$05*3
printstr=       kernel+$06*3
clrscr=         kernel+$07*3
getch=          kernel+$08*3
putch=          kernel+$09*3
getprstate=     kernel+$0a*3
setprstate=     kernel+$0b*3
getprsleepie=   kernel+$0c*3
setprsleepie=   kernel+$0d*3
gets=           kernel+$0e*3
puts=           kernel+$0f*3
reservestdright=kernel+$10*3
releasestdright=kernel+$11*3
workmode=       kernel+$12*3
idlemode=       kernel+$13*3
suicide=        kernel+$14*3
spawn=          kernel+$15*3
kill=           kernel+$16*3
getworkers=     kernel+$17*3
getidlers=      kernel+$18*3
getdead=        kernel+$19*3
phex16=         kernel+$1a*3
phex4=          kernel+$1b*3
horsepower=     kernel+$1c*3
csup=           kernel+$1d*3
cntarg=         kernel+$1e*3
seekarg=        kernel+$1f*3
dset=           kernel+$20*3
dopen=          kernel+$21*3
dclose=         kernel+$22*3
dget=           kernel+$23*3
dput=           kernel+$24*3
dsetread=       kernel+$25*3
dsetwrite=      kernel+$26*3
dinit=          kernel+$27*3
calstrlen2=     kernel+$28*3
dname=          kernel+$29*3
waitnokey=      kernel+$2a*3
 
*=$2000
;
; WILD CAT 0.0.1
;
.text "mz"
.word main ;Start jumpy address

main                    nop
                        ;
                        jsr reservestdright
                        ; Any arguments ?
                        ldx #<stdin
                        ldy #>stdin
                        jsr cntarg
                        cpx #2 ;  More args than just the fname?
                        bcc main_l1

                        ldx #<stdin
                        ldy #>stdin
                        lda #1          ; Seek to argument #2 beginning
                        jsr seekarg
                        stx $fb
                        sty $fc

                        ldx #<prgnote3
                        ldy #>prgnote3
                        jsr puts
                        ;
                        ldx $fb
                        ldy $fc
                        jsr puts
                        ;
                        lda #13
                        jsr putch

                        ;
                        lda #0
                        sta alts

                        ; Open file for reading
                        lda #$49
                        ldx #8
                        ldy #0
                        jsr dset
                        ldx $fb
                        ldy $fc
                        jsr calstrlen2  ; calstrlen2='Type2: Space=End
                        tya
                        ldx $fb
                        ldy $fc
                        jsr dname
                        jsr dopen
                        ldx #$49
                        jsr dsetread
                        ;
                        jsr stream2stdout
                        ;
                        lda #$49
                        jsr dclose
                        jsr dinit
                        ;
                        lda #13
                        jsr putch

                        ;
                        lda #$ff
                        sta alts

                        jmp endi
                        ;
main_l1                 nop
                        ldx #<prgnote
                        ldy #>prgnote
                        jsr puts
                        ;
endi                    jsr releasestdright
                        jmp suicide

prgnote                 .text "(wild)cat - file printting utility"
                        .byte 13
                        .text "program that is not aimed to clone unix"
                        .byte 13
                        .text "cat command, but instead to serve"
                        .byte 13
                        .text "basic needs of jtmos has for"
                        .byte 13
                        .text "the wild cat(kind of cat clone)."
                        .byte 13
                        .text "usage: cat [filename]"
                        .byte 13,0

prgnote2                .text "(wild)cat - ?file not found."
                        .byte 13,0

prgnote3                .text "searching for "
                        .byte 0

stream2stdout           lda #0
                        sta var1
s2stdout_l1             lda #$49
                        jsr dget
                        jsr putch
n40                     bit eof
                        bvc s2stdout_l1
stoppi                  rts
var1                    .byte 0
