;---------------------------------------------------------------------------
;
;       Multitasking for Commodore 64
;       "MultiThread handling system" version 0.0.5 (1999)
;
;---------------------------------------------------------------------------
;
kernel=$8000
;
*=kernel
deficallfreq=$0400
null=0
keybfreq=$100/8        ; Keyboard reading frequency($ea31/$ea7e)
version=$0100
stackl=$40 ; $00-$f0, size of the stack, lower=more, higher=less
           ; (Note: Stack grows backwards)
           ; With larger stacks it takes more time to switch
           ; between tasks, so think carefully do you really
           ; need huge $f0 byte f.e. stacks?
           ; $40 is actually quite nice value.
           ; Even on SCPU this really matters, even if
           ; task switching doesn't take time it anyway decreases
           ; the time that could be used more wisely.

; Memory region kernel-$1000 contains storage space for
; 16 processes stacks.
stack1=kernel-$1000  ; Virtual stack1 location in physical memory
stack2=kernel-$0f00  ; --------stack2--------
stack3=kernel-$0e00  ; --------stack3--------
stack4=kernel-$0d00  ; --------stack4--------
stack5=kernel-$0c00  ; --------stack5--------
stack6=kernel-$0b00  ; --------stack6--------
stack7=kernel-$0a00  ; --------stack7--------
stack8=kernel-$0900  ; --------stack8--------
stack9=kernel-$0800  ; --------stack9--------
stacka=kernel-$0700  ; --------stacka--------
stackb=kernel-$0600  ; --------stackb--------
stackc=kernel-$0500  ; --------stackc--------
stackd=kernel-$0400  ; --------stackd--------
stacke=kernel-$0300  ; --------stacke--------
stackf=kernel-$0200  ; --------stackf--------
stack10=kernel-$0100 ; --------stack10--------

; Task mode const's
dead=0
worker=1
idler=2

; REU I/O
reu_status   = $DF00
reu_command  = $DF01
reu_c64base  = $DF02
reu_reubase  = $DF04
reu_translen = $DF07
reu_irqmask  = $DF09
reu_control  = $DF0A

; SYSTEM VARIABLES (Critical!)
baswarm=$0302 ; Basic warm start address
kflag=$028d ; KEYBOARD FLAG
stdin=$9e00 ; STDIN - Keyboard input data
stderr=$9f00  ; STDERR - Errors buffer here
stderro=$9fff ; STDERRO - Buffer owner(Value=PID)
cpar=$03e0   ; C compiler support reserved!
power=$03f0  ; 32bit value that tells how much horse power
             ; MTASK has left to run new threads.
icallfreq=$3f4 ; Interrupt Calling Frequency
r0=$3d0 ; Virtual 32bit registers
r1=$3d4
r2=$3d8
r3=$3dc
pristd=$03fb ; PRIMARY STD CASING :
             ; If this variable is set then
             ; all STD routines will be executed
             ; in non task switchable mode.
             ; (If multitasking is slowing down one single process,
             ;  then it is possible to set this variable to
             ;  make screen output look smoother atleast).
stdright=$03fc ; Don't set manually!(0=Rights not reserved,
               ; !=thread id that has reserved the std IN/OUT rights.)
curtask=$03fd ; Currently running task
action=$03fe
alts=$03ff ;Allow task switch (0=False, 1=True)
delayv=16 ; This value sets the frequence of task scheduling,
          ; smaller value = quicker
          ; larger value = slower
          ; Quicker means that more CPU power will be reserved for
          ; task switching.(Default 16 is just fine)
          ; Too small values(like 1-8) may jam the processor on
          ; C64's without SCPU or any other speeder.
maxtasks=15 ; Valid values = 1,3,7,15,...
            ; 1=2, 3=4, 7=8, 15=16, .. (acts like AND)

;- #  KERNEL function calls ------------------------------------------------
                                                ; Offset Factor
main                    jmp install             ;00 * 3
                        jmp engine              ;01 ..
                        jmp irq                 ;02
                        jmp load                ;03 x,y
                        jmp save                ;04 x,y
                        jmp calstrlen           ;05 x,y
                        jmp printstr            ;06 x,y
                        jmp clrscr              ;07
                        jmp getch               ;08
                        jmp putch               ;09
                        jmp getprstate          ;0a x=PID, ret: a=state
                        jmp setprstate          ;0b a=state,x=PID
                        jmp getprsleepie        ;0c
                        jmp setprsleepie        ;0d
                        jmp gets                ;0e x,y
                        jmp puts                ;0f x,y
                        jmp reservestdright     ;10
                        jmp releasestdright     ;11
                        jmp workmode            ;12 Sets process in work mode
                        jmp idlemode            ;13 Sets process in idle mode
                        jmp suicide             ;14 Terminates process
                        jmp spawn               ;15 Create a new process(X,Y=OFFSET, Return value: X=PID)
                        jmp kill                ;16 Kill a process(X=PID)
                        jmp getworkers          ;17 Amount of worker jobs
                        jmp getidlers           ;18 ..        idler jobs
                        jmp getdead             ;19 ..        dead jobs
                        jmp phex16              ;1a Print 16bit hex value X:A
                        jmp phex4               ;1b Print 4bit hex value  X:A
                        jmp horsepower          ;1c Horsepower (Retval: X:Y = CPU power, requires process 'METER')
                        jmp csup                ;1d C language support
                        jmp cntarg              ;1e Counts amount of argrs in
                                                ;   a string
                        jmp seekarg             ;1f finds offset of
                                                ;   specified argument #
                                                ;   from the arg string.
                        jmp dset                ;20 dset (set init type)
                        jmp dopen               ;21 dopen (open stream)
                        jmp dclose              ;22 dclose (close stream)
                        jmp dget                ;23 dget (get stream)
                        jmp dput                ;24 dput (put stream)
                        jmp dsetread            ;25 dsetread (set stream=read)
                        jmp dsetwrite           ;26 dsetwrite (set stream=write)
                        jmp dinit               ;27 dinit (init I/O)
                        jmp calstrlen2          ;28 calstrlen2 (Type 2, uses space as end of string indetifier mark)
                        jmp dname               ;29 dname (sets filename)
                        jmp waitnokey           ;2a waitnokey(waits until no key have been pressed)
                        jmp getpridenty         ;2b getpridenty
                        jmp setpridenty         ;2c setpridenty
                        jmp movereutoram        ;2d movereutoram (REU support)
                        jmp moveramtoreu        ;2e moveramtoreu
                        .word stack1            ;
                        .word version           ;
;---------------------------------------------------------------------------
                        .text "( mtask )" ; Indentity string
;---------------------------------------------------------------------------
; Execution start addresses for 16 different threads.
threads                 .word meter
                        .word loadshell
                        .word idlepr
                        .word idlepr
                        .word idlepr
                        .word idlepr
                        .word idlepr
                        .word idlepr
                        .word idlepr
                        .word idlepr
                        .word idlepr
                        .word idlepr
                        .word idlepr
                        .word idlepr
                        .word idlepr
                        .word idlepr
                        ; Process states
                        ; (Value)
                        ; 0=Dead        (Not running 0 cr)
                        ; 1=Worker      (Mostly active 1/1 cr, works as
                        ;                background task)
                        ; 2=Idler       (Low CPU usage needed only, 1/8 cr)
                        ; ** Future expansion: **
                        ; 3=Screen      (Uses screen)
                        ; 4=Audio       (Uses audio)
                        ;
                        ; Note: 'cr' means 'call rate per hit'.
states                  .byte worker,idler,dead,dead
                        .byte dead,dead,dead,dead
                        .byte dead,dead,dead,dead
                        .byte dead,dead,dead,dead
                        ; (Note: Dead processes are also
                        ;  free entries for end processes,
                        ;  so don't allocate too many processes
                        ;  in beginning of boot up,
                        ;  so that shell and other applications
                        ;  can spawn enough many processes.
                        ;  Not leaving not enough entries free
                        ;  means also that the system will probaly crash
                        ;  or malfunction)
                        ;

                        ; This table is only used
                        ; with idler tasks.
sleepies                .byte $00,$10,$20,$30
                        .byte $40,$50,$60,$70
                        .byte $80,$90,$a0,$b0
                        .byte $c0,$d0,$e0,$f0

                        ; Processes indenty offsets,
                        ; null means no indenty available.
                        ; If specified entry not null then
                        ; specified entry points to a location
                        ; that has ASCIIZ(pet) based string
                        ; that is either name of the process
                        ; or something similair like that.
                        ;
identy                  .word id_meter,id_shldr,null,null
                        .word null,null,null,null
                        .word null,null,null,null
                        .word null,null,null,null
;---------------------------------------------------------------------------
id_meter                .text "cpu meter"
                        .byte 0

id_shldr                .text "shell loader"
                        .byte 0
;---------------------------------------------------------------------------
;                       - X = ENTRY*2
getpridenty             lda identy,x ; NOTICE!: X must be multiplied by 2 !
                        pha
                        lda identy+1,x
                        tay
                        pla
                        tax
                        rts
;---------------------------------------------------------------------------
; A:Y(X)                - A:Y = NEW OFFSET      (Y=HI, A=LO)
;                         X = SPECIFIED ENTRY *2 !
setpridenty             sta identy,x
                        tya
                        sta identy+1,x
                        rts
;---------------------------------------------------------------------------
setprstate              sta states,x
                        rts
;---------------------------------------------------------------------------
getprstate              lda states,x
                        rts
;---------------------------------------------------------------------------
setprsleepie            sta sleepies,x
                        rts
;---------------------------------------------------------------------------
getprsleepie            lda sleepies,x
                        rts
;---------------------------------------------------------------------------
getch                   jsr $ffe4
                        beq getch
                        rts
;---------------------------------------------------------------------------
putch                   bit pristd
                        bvc putchl1
                        ldx alts
                        stx putchtmp
                        ldx #0
                        stx alts
                        jsr $ffd2
                        ldx putchtmp
                        stx alts
                        rts
putchl1                 jmp $ffd2
putchtmp                .byte 0
;---------------------------------------------------------------------------
fmsg                    .byte 13
                        .text "jtmos kernel (mos6510) unoptimized"
                        .byte 13
                        .text " multitasking upto 16 threads enabled"
                        .byte 13
                        .text " $7000-$7fff 4K processes stacks"
                        .byte 13
                        .text " $8000-$8fff 4K kernel"
                        .byte 13,13,13
                        .byte 0
;---------------------------------------------------------------------------        
install                 sei
                        cld
                        ldx #$fb
                        txs
                        lda #$37
                        sta 1
                        lda #$31
                        ldx #$ea
                        sta $0314
                        stx $0315
                        lda #11
                        sta $d020
                        sta $d021
                        lda #$1b
                        sta $d011
                        lda #$15
                        sta $d018
                        lda #$c8
                        sta $d016
                        lda #$00
                        sta $d418
                        lda $dd00
                        ora #3
                        sta $dd00
                        lda #4
                        sta $0288
                        lda #1
                        sta $0286
                        lda #0
                        sta curtask
                        ldy #0
                        ldx #0
                        clc
                        jsr $fff0
                        lda #147 ; CLRSCR
                        jsr $ffd2
                        ;
                        lda #<deficallfreq
                        ldx #>deficallfreq
                        sta icallfreq
                        stx icallfreq+1
                        ;
                        lda icallfreq
                        sta $dc04
                        lda icallfreq+1
                        sta $dc05
                        ;
                        lda #0
                        sta stderr
                        ;
                        lda #0
                        sta pristd
                        cli
                        ;
                        ldx #<fmsg
                        ldy #>fmsg
                        jsr printstr
                        ;
                        jmp engine
;---------------------------------------------------------------------------
printstr                stx $30
                        sty $31
printstr_l1             ldy #0
                        lda ($30),y
                        beq printstr_l2
                        jsr putch
                        jsr i30
                        jmp printstr_l1
printstr_l2             rts
;---------------------------------------------------------------------------
textcolor               sta 646
                        rts
;---------------------------------------------------------------------------
clrscr                  lda #1
                        jsr textcolor
                        lda #147
                        jmp putch
;---------------------------------------------------------------------------
; This routine initializes specified stack.
initstack               sei                        
                        clc
                        adc #>stack1
                        sta $31
                        lda #0
                        sta $30
                        ; Save current stack
                        tsx
                        stx tmp
                        ldx #0
initstack_l1            lda $0100,x
                        sta tmp+1,x
                        inx
                        bne initstack_l1
                        ; Setup copy stack
                        ldx #$fb
                        txs
                        ; Offset16 + FlagReg.
                        lda #>execthread
                        pha
                        lda #<execthread
                        pha
                        ; FlagReg!
                        php
                        pla
                        and #$ff-4 ; FlagReg:Interrupts allowed
                        pha
                        ; Three registers a,x,y
                        lda #0
                        pha
                        lda #0
                        pha
                        lda #0
                        pha
                        ;
                        ldy #STACKL
initstack_l2            lda $0100,y
                        sta ($30),y
                        iny
                        bne initstack_l2
                        tsx
                        txa
                        ldy #0
                        sta ($30),y
                        ; Restore previous stack
                        ldx #0
initstack_l3            lda tmp+1,x
                        sta $0100,x                        
                        inx
                        bne initstack_l3
                        ldx tmp
                        txs
                        rts
                        ; Note: This routine won't restore interrupt mask,
                        ;       thats because the routine that calls it
                        ;       probaly expects this routine to act like
                        ;       it does.(No CLI opcode)
;---------------------------------------------------------------------------
; This code starts MTASK ...
engine                  sei
                        ;--------------------------------------
                        ; Init stacks
                        lda #0
                        sta 2
engine_l1b              lda 2
                        jsr initstack
                        inc 2
                        lda 2
                        cmp #$10
                        bne engine_l1b
                        ;
                        lda #<irq ; Our own IRQ
                        ldx #>irq
                        sta $0314
                        stx $0315
                        lda #0
                        sta $d012
                        ;
                        lda #255 ; Allow multitasking
                        sta alts
                        ;
                        lda #255 ; Allways be virgin at beginning.
                        sta virgin 
                        ;
                        lda #delayv
                        sta delay
                        ;
                        jsr inssigbrk
                        jsr insbaswarm
                        ;
                        cli
ul                      jmp ul
;---------------------------------------------------------------------------
virgin                  .byte $ff
;---------------------------------------------------------------------------
insbaswarm              sei
                        lda #<baswarmh
                        ldx #>baswarmh
                        sta baswarm
                        stx baswarm+1
                        rts
;---------------------------------------------------------------------------
baswarmh                jmp install
;---------------------------------------------------------------------------
inssigbrk               sei
                        lda #<sigbrk
                        ldx #>sigbrk
                        sta $0316
                        stx $0317
                        rts
;---------------------------------------------------------------------------
sigbrk                  nop
                        ; Clear BRK flag to prevent BRK signal
                        ; occuring again.
                        php
                        pla
                        and #$ff-16
                        pha
                        plp
                        ; Kill current task(The bugging task)
                        ldx curtask
                        jsr kill
                        ; Order task switch immediatly
                        jsr immswitch
                        ; Put error msg at STDERR.
                        ldx #<sigbrktxt
                        ldy #>sigbrktxt
                        jsr puts2stderr
                        ; Do it now to prevent confusing application to
                        ; crash the system.
                        jmp irq
;---------------------------------------------------------------------------
sigbrktxt               .text "?process #curtask caused sigbrk - process terminated."
                        .byte 13
                        .byte 0
;---------------------------------------------------------------------------
; This function prints specified ASCIIZ(PET) text into STDERR buffer.
; Note: STDERR buffer can handle one string per time.
puts2stderr             stx $30
                        sty $31
                        lda #0 ; MAX STDERR LENGTH
                        sta $32
puts2stderr_l1          ldy #0
                        lda ($30),y
                        beq puts2stderr_l2
                        ldy $32
                        sta stderr,y
                        ; Count max. len
                        inc $32
                        lda $32
                        cmp #80
                        beq puts2stderr_l2
                        ; Increase pointer
                        jsr i30
                        ; Loop ..
                        jmp puts2stderr_l1
puts2stderr_l2          ldy $32
                        lda #0
                        sta stderr,y
                        rts
;---------------------------------------------------------------------------
; Note:
; This function is only used by the KERNEL and should not be
; used by any application !
;
execthread              ldx curtask
                        lda #"."
;                       sta $0400+(5*40),x
                        lda curtask
                        and #maxtasks
                        asl
                        tax
                        lda threads+0,x
                        sta execthread_ad+1
                        lda threads+1,x
                        sta execthread_ad+2
execthread_ad           jmp $beef
;---------------------------------------------------------------------------
loadshell               nop
                        ;
                        lda curtask
                        asl
                        tax
                        lda #<id_shldr
                        ldy #>id_shldr
                        jsr setpridenty
                        ;
                        lda #0
                        sta $9d
                        ldx #<fname
                        ldy #>fname
                        jsr load
                        bcc loadsh_ok
                        ldx #<loadshell_txt
                        ldy #>loadshell_txt
                        jsr printstr
                        jsr getch
                        jmp loadshell
                        ;OK:
loadsh_ok               lda #6
                        sta $d020
                        sta $d021
                        jmp ($0800)

loadshell_txt           .text "error while loading shell - insert diskette with shell program and press anykey to continue, thank you('shell.run')"
                        .byte 13,0

fname                   .text "shell.run"
                        .byte 0
delay                   .byte 0
timer                   .word 0,0
;
inctimer                inc timer
                        bne inctimer_l1
                        inc timer+1
inctimer_l1             rts
;---------------------------------------------------------------------------
irq                     nop
                        ;
                        jsr inctimer
                        ;
                        dec delay
                        beq irq_l0
                        jmp irq_lm1
irq_l0                  lda #delayv
                        sta delay
                        ;
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
tswitch                 inc action
                        bit alts ; Allow Process Switch (ALTS)
                        bvs irq_led4
                        ; Task Switch Not allowed!
                        jmp irq_l4
irq_led4                nop ; Task Switch allowed ..
                        ;
                        ; Idler process?
                        ldx curtask
                        jsr getprstate

                        ; Next task
irq_next                inc curtask
                        lda curtask
                        and #maxtasks
                        sta curtask
                        ; Idler?
                        ldx curtask
                        jsr getprstate
                        cmp #idler
                        bne irq_l5
                        ; Shall we idle a bit.
irq_idl1                jsr getprsleepie
                        clc
                        adc #$100/8
                        jsr setprsleepie
                        bcs irq_l5 ; Idled already enough to
                                   ; have right to work abit?
                        jmp irq_next ; Nope, let others work!
                        ;
                        ; Dead thread?
irq_l5                  ldx curtask
                        jsr getprstate
                        cmp #dead
                        bne irq_l6
                        ; Skip all dead processes.
                        jmp irq_next
                        ; Setup new process stack pointer
irq_l6                  lda curtask
                        clc
                        adc #>stack1
                        sta $41
                        lda #$00
                        sta $40
                        ;
                        ;
                        bit virgin
                        bvc irq_l2
                        lda #0
                        sta virgin
                        ldy #STACKL ;Note: Cool Optimization 8-)
irq_l3                  lda ($40),y
                        sta $0100,y
                        iny
                        lda ($40),y
                        sta $0100,y
                        iny
                        lda ($40),y
                        sta $0100,y
                        iny
                        lda ($40),y
                        sta $0100,y
                        iny
                        bne irq_l3
                        jmp irq_skip
irq_l2                  nop
                        ; Store old stack pointer
                        ; 
                        tsx
                        txa
                        ldy #0
                        sta ($44),y
                        ;
                        ; Save old stack
                        ; Store new stack (16::$42 points at old stack)
                        ldy #STACKL
irq_l1                  lda $0100,y
                        sta ($44),y
                        lda ($40),y
                        sta $0100,y
                        iny
                        lda $0100,y
                        sta ($44),y
                        lda ($40),y
                        sta $0100,y
                        iny
                        lda $0100,y
                        sta ($44),y
                        lda ($40),y
                        sta $0100,y
                        iny
                        lda $0100,y
                        sta ($44),y
                        lda ($40),y
                        sta $0100,y
                        iny
                        bne irq_l1
irq_skip                nop
                        ;
                        ;
                        lda $40
                        ldx $41
                        sta $44
                        stx $45
                        ;
                        ldy #0 ; Set new stack pointer
                        lda ($40),y
                        tax
                        txs
                        ;
irq_l4                  nop
irq_lm1                 nop
;inc $d019              lda jdel
                        clc
                        adc #keybfreq
                        sta jdel
                        bcs jmpea31
                        jmp $ea7e
jmpea31                 jmp $ea31
jdel                    .byte 0,0
;------------------------------------------------------------------------
phex16                  tay
                        txa
                        pha     ;(x:a)=16bit value
                        tya     ;^h^l
                        pha
                        lsr
                        lsr
                        lsr
                        lsr
                        jsr phex4
                        pla
                        and #$0f
                        jsr phex4
                        pla
                        pha
                        lsr
                        lsr
                        lsr
                        lsr
                        jsr phex4
                        pla
                        and #$0f
                        jsr phex4
                        rts
                        ;
phex4                   and #$0f
                        tax
                        lda hextab,x
                        jmp putch
                        
hextab                  .text "0123456789abcdef"


d30                     lda $30
                        sec
                        sbc #1
                        sta $30
                        lda $31
                        sbc #0
                        sta $31
                        rts

i30                     inc $30
                        bne b30
                        inc $31
b30                     rts
;---------------------------------------------------------------------------
calstrlen               stx $4e
                        sty $4f
                        ldy #0
calstrlen_l1            lda ($4e),y
                        beq calstrlen_l2
                        iny
                        bne calstrlen_l1
calstrlen_l2            rts
;---------------------------------------------------------------------------
calstrlen2              stx $4e ; Acts like calstrlen but
                                ; doesn't use asciiz(pet),
                                ; but instead strings that end with
                                ; 'space' character(0x20, 32, ' ').
                        sty $4f
                        ldy #0
calstrlen2_l1           lda ($4e),y
                        cmp #" "
                        beq calstrlen2_l2
                        cmp #0
                        beq calstrlen2_l2
                        iny
                        bne calstrlen2_l1
calstrlen2_l2           rts
;---------------------------------------------------------------------------
; Following two functions turns off multitasking to prevent system crashing
; or something similiar to happend. 
;---------------------------------------------------------------------------
load                    stx $30
                        sty $31
                        ; Turn multitasking Off(Push old value)
                        lda alts
                        pha
                        lda #0
                        sta alts
                        ; Calculate string length
                        jsr calstrlen
                        sty $32
                        ;
                        lda #8
                        tax
                        tay
                        jsr $ffba
                        lda $32
                        ldx $30
                        ldy $31
                        jsr $ffbd
                        lda #0
                        jsr $ffd5
                        ;
                        pla             ;Restore ALTS previous state.
                        sta alts
                        ;
                        rts
;---------------------------------------------------------------------------
save                    sta save_ad1+1
                        stx $30
                        sty $31
                        ; Turn multitasking Off
                        lda alts
                        pha
                        lda #0
                        sta alts
                        ; Calculate string length
                        jsr calstrlen
                        sty $32
                        ;
                        lda #8
                        ldx $ba
                        tay
                        jsr $ffba
                        lda $32
                        ldx $30
                        ldy $31
                        jsr $ffbd
save_ad1                lda #$fb
                        jsr $ffd8
                        ;
                        pla
                        sta alts
                        ;
                        rts
;---------------------------------------------------------------------------
flashscreen             inc $d020
                        rts
;---------------------------------------------------------------------------
; This routine idles. (Well, it doesn't take too much CPU power that way 8-)
;
idlepr                  jsr immswitch
                        jmp idlepr

;---------------------------------------------------------------------------
immswitch               lda #1
                        sta delay
                        rts
;---------------------------------------------------------------------------
reservestdright         sei
                        ldx #0
                        cpx stdright
                        bne reservestdri_l1
                        ldx curtask
                        stx stdright
                        lda #0 ; Means no error.
                        cli
                        rts
reservestdri_l1         lda #$ff ; Means error.
                        cli
                        rts
;---------------------------------------------------------------------------
;Note: Process that doesn't currently own the right MUST NOT attempt
;to release other processes STDOUT rights.
releasestdright         sei
                        ldx curtask
                        cpx stdright
                        bne releasestdri_l1
                        lda #0
                        sta stdright
                        cli
                        rts
releasestdri_l1         lda #$ff
                        cli
                        rts
;---------------------------------------------------------------------------
gets                    stx $30 ; Yes, apps shouldn't use sametime $30,$31!
                        sty $31
                        ldy #0
gets_l1                 jsr $ffcf
                        cmp #13
                        beq gets_l2
                        sta ($30),y
                        iny
                        bne gets_l1
gets_l2                 lda #0
                        sta ($30),y
                        rts                        
;---------------------------------------------------------------------------
puts                    jmp printstr
;---------------------------------------------------------------------------
workmode                ldx curtask
                        lda #worker
                        jmp setprstate
;---------------------------------------------------------------------------
idlemode                ldx curtask
                        lda #idler
                        jmp setprstate
;---------------------------------------------------------------------------
; When process calls this function it makes suicide.
; (System stops running specified process when it is set to dead mode)
suicide                 ldx curtask
                        jsr kill
suicidel1               nop
                        jmp suicidel1
;---------------------------------------------------------------------------
findfreepr              nop ; Finds free process, returns A=FF if not found.
                        ldx #maxtasks
findfreepr_l1           jsr getprstate
                        cmp #dead
                        beq findfreepr_l2
                        dex
                        bpl findfreepr_l1
                        lda #$ff ; Error if no free entries.
                        rts
findfreepr_l2           lda #0
                        rts
;---------------------------------------------------------------------------
getdead                 nop             ; Retval: Y=AMOUNT
                        ldx #maxtasks
                        ldy #0
getdead_l1              jsr getprstate
                        cmp #dead
                        bne getdead_l2
                        iny
getdead_l2              dex
                        bpl getdead_l1
                        rts
;---------------------------------------------------------------------------
getworkers              nop             ; Retval: Y=AMOUNT
                        ldx #maxtasks
                        ldy #0
getworker_l1            jsr getprstate
                        cmp #worker
                        bne getworker_l2
                        iny
getworker_l2            dex
                        bpl getworker_l1
                        rts
;---------------------------------------------------------------------------
getidlers               nop             ; Retval: Y=AMOUNT
                        ldx #maxtasks
                        ldy #0
getidlers_l1            jsr getprstate
                        cmp #idler
                        bne getidlers_l2
                        iny
getidlers_l2            dex
                        bpl getidlers_l1
                        rts
;---------------------------------------------------------------------------
spawn                   txa             ; x,y = offset to new process start
                        pha
                        tya
                        pha
                        jsr findfreepr
                        beq spawn_l1
                        pla
                        pla
                        lda #$ff
                        ; Error
                        rts
spawn_l1                nop ; no error
                        sei
                        stx spawn_pid
                        txa
                        asl
                        tax
                        pla
                        sta threads+1,x
                        pla
                        sta threads,x
                        ;
                        lda spawn_pid
                        jsr initstack
                        ;
                        ; Wake up process
                        ldx spawn_pid
                        lda #worker
                        jsr setprstate
                        ;
                        ; No error (A=0, X=PID #)
                        ; Return child PID to parent.
                        ldx spawn_pid
                        cli
                        lda #0
                        rts
spawn_pid               .byte 0
;---------------------------------------------------------------------------
kill                    lda #dead                ; x=PID to kill
                        jmp setprstate
;---------------------------------------------------------------------------
horsepower              ldx meter_v
                        ldy meter_v+1
                        rts
;---------------------------------------------------------------------------
makejmp                 jmp (makejmp_p1)
makejmp_p1              .word 0
;---------------------------------------------------------------------------
csup                    lda cpar+0
                        ldx cpar+1
                        sta makejmp_p1
                        stx makejmp_p1+1
                        ldx cpar+2
                        ldy cpar+3
                        lda cpar+4
                        jmp makejmp
;---------------------------------------------------------------------------
; This function counts arguments in a string. (Retval: Register X = Count)
cntarg                  stx $30
                        sty $31
                        ldy #0
                        ldx #0
cntarg_l1               lda ($30),y
                        sta $33
                        cmp #0
                        beq cntarg_l2
                        cmp #" "
                        bne cntarg_l4
                        lda #1
                        sta cntarg_spc
                        jmp cntarg_l3
                        ;------------
cntarg_l4               lda #0
                        sta cntarg_spc
                        ;------------
cntarg_l3               nop
                        ; Case: Offset 0 will be ignore in compare.
                        lda $33
                        cmp #" "
                        beq cntarg_z
                        cpy #0
                        beq cntarg_z
                        lda cntarg_spc
                        cmp cntarg_spcl
                        beq cntarg_z
                        ; Increase arg. count when it seems reasonable.
                        inx
cntarg_z                nop
                        ;
                        lda cntarg_spc
                        sta cntarg_spcl
                        ;
                        ; Max.str len = 255, if y=0 then exit.
                        iny
                        beq cntarg_l2
                        jmp cntarg_l1
cntarg_l2               cpy #0
                        beq cntarg_zap
                        inx
cntarg_zap              rts
                        ;
cntarg_spcl             .byte 0 ; Last space value
cntarg_spc              .byte 0 ; Space value
;---------------------------------------------------------------------------
; 
seekarg                 sta $32
                        stx $30
                        sty $31
                        ldy #0
                        ldx #0
seekarg_l1              lda ($30),y
                        sta $33
                        cmp #0
                        beq seekarg_l2
                        cmp #" "
                        bne seekarg_l4
                        lda #1
                        sta seekarg_spc
                        jmp seekarg_l3
                        ;------------
seekarg_l4              lda #0
                        sta seekarg_spc
                        ;------------
seekarg_l3              nop
                        ; Case: Offset 0 will be ignore in compare.
                        lda $33
                        cmp #" "
                        beq seekarg_z
                        cpy #0
                        beq seekarg_z
                        lda seekarg_spc
                        cmp seekarg_spcl
                        beq seekarg_z
                        ; Increase arg. count when it seems reasonable.
                        inx
                        cpx $32
                        beq seekarg_l2
seekarg_z               nop
                        ;
                        lda seekarg_spc
                        sta seekarg_spcl
                        ;
                        ; Max.str len = 255, if y=0 then exit.
                        iny
                        beq seekarg_l2
                        jmp seekarg_l1
seekarg_l2              cpy #0
                        beq seekarg_zap
                        inx
seekarg_zap             tya             ; Offset+$30:$31=Final offset
                        clc
                        adc $30
                        sta $30
                        lda $31
                        adc #0
                        sta $31
                        txa             ; Argument #
                        ldx $30         ; offset l
                        ldy $31         ; offset h
                        rts
seekarg_spcl            .byte 0 ; Last space value
seekarg_spc             .byte 0 ; Space value
;---------------------------------------------------------------------------
waitnokey               jsr getch
                        bne waitnokey
                        rts
;---------------------------------------------------------------------------










; ******** FILE/STREAM I/O HANDLING ***********
;---------------------------------------------------------------------------
dtmp                    .word 0
;---------------------------------------------------------------------------
dopen                   sta dtmp
                        lda alts
                        pha
                        lda #0
;                       sta alts
                        lda dtmp
                        jsr $ffc0
                        pla
                        sta alts
                        rts
;---------------------------------------------------------------------------
dclose                  sta dtmp
                        lda alts
                        pha
                        lda #0
;                       sta alts
                        lda dtmp
                        jsr $ffc3
                        pla
                        sta alts
                        rts
;---------------------------------------------------------------------------
dget                    jmp $ffcf
                        sta dtmp
                        lda alts
                        pha
                        lda #0
;                       sta alts
                        lda dtmp
                        jsr $ffcf
                        pla
                        sta alts
                        rts
;---------------------------------------------------------------------------
dput                    sta dtmp
                        lda alts
                        pha
                        lda #0
;                       sta alts
                        lda dtmp
                        jsr $ffd2
                        pla
                        sta alts
                        rts
;---------------------------------------------------------------------------
dset                    sta dtmp
                        lda alts
                        pha
                        lda #0
;                       sta alts
                        lda dtmp
                        jsr $ffba
                        pla
                        sta alts
                        rts
;---------------------------------------------------------------------------
dinit                   sta dtmp
                        lda alts
                        pha
                        lda #0
;                       sta alts
                        lda dtmp
                        jsr $ffcc
                        pla
                        sta alts
                        rts
;---------------------------------------------------------------------------
dsetread                sta dtmp
                        lda alts
                        pha
                        lda #0
;                       sta alts
                        lda dtmp
                        jsr $ffc6
                        pla
                        sta alts
                        rts
;---------------------------------------------------------------------------
dsetwrite               sta dtmp
                        lda alts
                        pha
                        lda #0
;                       sta alts
                        lda dtmp
                        jsr $ffc9
                        pla
                        sta alts
                        rts
;---------------------------------------------------------------------------
dname                   sta dtmp
                        lda alts
                        pha
                        lda #0
;                       sta alts
                        lda dtmp
                        jsr $ffbd
                        pla
                        sta alts
                        rts
;---------------------------------------------------------------------------









;----------------------------------------------------------------------------
;
;       JTT programming language REU support library
;       provides access to Ram Expansion Unit memory 
;       when using JTT.
;       (C)1998 by Jari Tapio Tuominen, all rights reserved.
;
; moveramtoreu
; movereutoram
;----------------------------------------------------------------------------


;----------------------------------------------------------------------------
;
;       moveramtoreu
;       ------------
;
;       Usage:
;
;       moveramtoreu( offsetreu offsetram length );
;
;       See: movereutoram
;
;----------------------------------------------------------------------------
moveramtoreu    lda #0
                sta reu_control ; to make sure both addresses are counted up
                ;
                lda r1+0
                sta reu_c64base
                lda r1+1
                sta reu_c64base + 1
                ;
                lda r0+0
                sta reu_reubase
                lda r0+1
                sta reu_reubase + 1
                lda r0+2
                sta reu_reubase + 2
                ;
                lda r2
                sta reu_translen
                lda r2 + 1
                sta reu_translen + 1
                lda #%10010000;  c64 -> REU with immediate execution
                sta reu_command
                rts

;----------------------------------------------------------------------------
;
;       movereutoram
;       ------------
;
;       Usage:
;
;       movereutoram( offsetreu offsetram length );
;
;       See: moveramtoreu
;
;----------------------------------------------------------------------------
movereutoram    lda #0
                sta reu_control ; to make sure both addresses are counted up
                ;
                lda r1+0
                sta reu_c64base
                lda r1+1
                sta reu_c64base + 1
                ;
                lda r0+0
                sta reu_reubase
                lda r0+1
                sta reu_reubase + 1
                lda r0+2
                sta reu_reubase + 2
                ;
                lda r2
                sta reu_translen
                lda r2 + 1
                sta reu_translen + 1
                lda #%10010001;  REU -> c64 with immediate execution
                sta reu_command
                rts
;----------------------------------------------------------------------------












;---------------------------------------------------------------------------
; This routine calculates amount of the CPU power left.
;
meter                   nop
                        ;
                        jsr idlemode
                        ;
                        lda curtask
                        asl
                        tax
                        lda #<id_meter
                        ldy #>id_meter
                        jsr setpridenty
                        ;
meter_l0                lda #0
                        sta timer
                        sta timer+1
                        sta meter_power
                        sta meter_power+1
                        sta meter_power+2
                        sta meter_power+3
meter_l1                jsr incmeter
                        ldx timer+1
                        cpx #1
                        bcc meter_l1
                        ;
                        lda meter_power
                        ldx meter_power+1
                        sta meter_v
                        stx meter_v+1
                        ;
                        jmp meter_l0

meter_v                 .word 0,0,0,0

incmeter                inc meter_power
                        bne incmeter_b1
                        inc meter_power+1
                        bne incmeter_b1
                        inc meter_power+2
                        bne incmeter_b1
                        inc meter_power+3
incmeter_b1             rts

meter_power             .word 0

;****************************************************************


tmp                     .byte 0 ; Temporary data storage area
                                ; (Must be located at end of all data).

