;--------------------------------------------------------------------------
;
;       PETIO.ASM
;       ---------
;
;      *** PETASCII I/O routines for Tapio language C-64 module ***
;       (C)1998 by Jari Tapio Tuominen, all rights reserved.
;
;--------------------------------------------------------------------------

;--------------------------------------------------------------------------
; Functions :
; (Name)                (Belongs to)            (ECPU registers)
;--------------------------------------------------------------------------
; ** Keyboard(/mouse), joystick and screen functions **
; putc11                System                  None
; print                 User                    r0=offset
; getch                 User                    r0=key pressed
; putch                 User                    r0=mark to output
; jgetchim              User                    r0=key pressed (0 if none)
; input                 User                    r0=Address to input store data
;                                                  (offset) f.e. "inputdata&"
;

; ** Disk functions **
; fopen                 User                    r0=handle r1=drive r2=?
;                                               r3=File name(offset)
; fload                 User                    r0=? r1=drive r2=?
;   \-> Uses $0330 kernal loader                r0=File name(offset)
; fdrive                User                    r0=Drive number(8-11)
; fclose                User                    r0=handle
; fioinit               User                    None
; fputc                 User                    r0=data
; fgetc                 User                    r0=handle, returns: r0=data
; fcreat                User                    r0=handle r1=drive r2=?
;                                               r3=File name(offset)
;
; Notice: All strings are (PC)ASCIIZ/(64)PETASCIIZ based,
;         so f.e. if you're planning to load some file in memory with fload,
;         then you should first produce an ASCIIZ or PETASCIIZ based string.
;
; (PET)ASCIIZ string example : "Hello world!",0
;
;--------------------------------------------------------------------------
putcbios rts
;--------------------------------------------------------------------------
putc11  jmp $ffd2
;--------------------------------------------------------------------------
;Prints asciiz string to stanard output
print   lda r0
        ldx r0+1
        sta $30
        stx $31
l000    ldy #0
        lda ($30),y
        cmp #0
        beq l002
        jsr putc11
        inc $30
        bne l001
        inc $31
l001    jmp l000
l002    rts
;--------------------------------------------------------------------------
getch   jsr $ffe4
        beq getch
        sta r0
        lda #0
        sta r0+1
        rts
;--------------------------------------------------------------------------
putch   lda r0
        jmp $ffd2
;--------------------------------------------------------------------------
jgetchim jsr $ffe4
        sta r0
        rts
;--------------------------------------------------------------------------
jgetfna stx $30       ;Hunts lenght of ASCIIZ based string and returns it
        sty $30+1
        ldy #0
jgetfl1 lda ($30),y
        cmp #0
        beq jgetfl2
        iny
        bne jgetfl1
jgetfl2 tya
        ldx $30
        ldy $30+1
        rts
;--------------------------------------------------------------------------
input   lda r0
        ldx r0+1
        sta $30
        stx $31
        ldy #0
inputl1 jsr $ffcf
        cmp #13
        beq inputl2
        sta ($30),y
        iny
        bne inputl1
inputl2 lda #0
        sta ($30),y
        sty $c8
        rts
;--------------------------------------------------------------------------
; *** Disk functions ***
;--------------------------------------------------------------------------
fopen   lda r0
        ldx r1
        ldy r2
        jsr $ffba
        ldx r3
        ldy r3+1
        jsr jgetfna
        jsr $ffc0
        ldx r0
        jmp $ffc6
;--------------------------------------------------------------------------
fload   lda #8
        ldx $ba
        tay
        jsr $ffba
        ldx r0
        ldy r0+1
        jsr jgetfna
        jmp $ffd5
;--------------------------------------------------------------------------
fdrive  lda r0
        sta $ba
        rts
;--------------------------------------------------------------------------
fclose  lda r0
        jmp $ffc3
;--------------------------------------------------------------------------
fioinit jmp $ffcc
;--------------------------------------------------------------------------
fputc   jmp $ffd2
;--------------------------------------------------------------------------
fgetc   lda r0
        jmp $f13e
;--------------------------------------------------------------------------
fcreat  nop
        lda r3
        ldx r3+1
        sta $30
        stx $31
        ldy #0
fcreatl lda ($30),y
        cmp #0
        beq fcreatz
        iny
        bne fcreatl
fcreatz lda #","
        sta ($30),y
        iny
        lda #"w"
        sta ($30),y
        iny
        lda #","
        sta ($30),y
        iny
        lda #"p"
        sta ($30),y
        iny
        lda #0
        sta ($30),y
        lda r0
        ldx r1
        ldy r2
        jsr $ffba
        ldx r3
        ldy r3+1
        jsr jgetfna
        jsr $ffc0
        ldx r0
        jmp $ffc9
;--------------------------------------------------------------------------
