;
; MTASK's Include File for Applications v0.0.1
;
version=$0100
stackl=$40 ; $00-$f0, size of the stack, lower=more, higher=less
           ; (Note: Stack grows backwards)
           ; With larger stacks it takes more time to switch
           ; between tasks, so think carefully do you really
           ; need huge $f0 byte f.e. stacks?
           ; $40 is actually quite nice value.
           ; Even on SCPU this really matters, even if
           ; task switching doesn't take time it anyway decreases
           ; the time that could be used more wisely.

; Memory region $7000-$7fff contains storage space for
; 16 processes stacks.
stack1=$7000  ; Virtual stack1 location in physical memory
stack2=$7100  ; --------stack2--------
stack3=$7200  ; --------stack3--------
stack4=$7300  ; --------stack4--------
stack5=$7400  ; --------stack5--------
stack6=$7500  ; --------stack6--------
stack7=$7600  ; --------stack7--------
stack8=$7700  ; --------stack8--------
stack9=$7800  ; --------stack9--------
stacka=$7900  ; --------stacka--------
stackb=$7a00  ; --------stackb--------
stackc=$7b00  ; --------stackc--------
stackd=$7c00  ; --------stackd--------
stacke=$7d00  ; --------stacke--------
stackf=$7e00  ; --------stackf--------
stack10=$7f00 ; --------stack10--------

; Task mode const's
dead=0
worker=1
idler=2
null=0

; SYSTEM VARIABLES (Critical!)
kflag=$028d ; KEYBOARD FLAG
stdin=$9e00 ; STDIN buffer
stderr=$9f00 ; STDERR - Errors buffer here
cpar=$03e0   ; C compiler support reserved!
power=$03f0  ; 32bit value that tells how much horse power
             ; MTASK has left to run new threads.
pristd=$03fb ; PRIMARY STD CASING :
             ; If this variable is set then
             ; all STD routines will be executed
             ; in non task switchable mode.
             ; (If multitasking is slowing down one single process,
             ;  then it is possible to set this variable to
             ;  make screen output look smoother atleast).
stdright=$03fc ; Don't set manually!(0=Rights not reserved,
               ; !=thread id that has reserved the std IN/OUT rights.)
curtask=$03fd ; Currently running task
action=$03fe
alts=$03ff ;Allow task switch (0=False, 1=True)
delayv=16 ; This value sets the frequence of task scheduling,
          ; smaller value = quicker
          ; larger value = slower
          ; Quicker means that more CPU power will be reserved for
          ; task switching.(Default 16 is just fine)
          ; Too small values(like 1-8) may jam the processor on
          ; C64's without SCPU or any other speeder.
maxtasks=15 ; Valid values = 1,3,7,15,...
            ; 1=2, 3=4, 7=8, 15=16, .. (acts like AND)


; Base Kernel
kernel=   $8000
; Kernel Calls
install=        kernel+$00*3
engine=         kernel+$01*3
irq=            kernel+$02*3
load=           kernel+$03*3
save=           kernel+$04*3
calstrlen=      kernel+$05*3
printstr=       kernel+$06*3
clrscr=         kernel+$07*3
getch=          kernel+$08*3
putch=          kernel+$09*3
getprstate=     kernel+$0a*3
setprstate=     kernel+$0b*3
getprsleepie=   kernel+$0c*3
setprsleepie=   kernel+$0d*3
gets=           kernel+$0e*3
puts=           kernel+$0f*3
reservestdright=kernel+$10*3
releasestdright=kernel+$11*3
workmode=       kernel+$12*3
idlemode=       kernel+$13*3
suicide=        kernel+$14*3
spawn=          kernel+$15*3
kill=           kernel+$16*3
getworkers=     kernel+$17*3
getidlers=      kernel+$18*3
getdead=        kernel+$19*3
phex16=         kernel+$1a*3
phex4=          kernel+$1b*3
horsepower=     kernel+$1c*3
csup=           kernel+$1d*3
cntarg=         kernel+$1e*3
seekarg=        kernel+$1f*3
dset=           kernel+$20*3
dopen=          kernel+$21*3
dclose=         kernel+$22*3
dget=           kernel+$23*3
dput=           kernel+$24*3
dsetread=       kernel+$25*3
dsetwrite=      kernel+$26*3
dinit=          kernel+$27*3
calstrlen2=     kernel+$28*3
dname=          kernel+$29*3
waitnokey=      kernel+$2a*3
getpridenty=    kernel+$2b*3
setpridenty=    kernel+$2c*3

;
; PS 0.0.1
;
*=$2000
.text "mz"
.word main

id_ps                   .text "ps"
                        .byte 0

main                    nop
                        ;
                        jsr reservestdright

                        ; Set identy of PS so the PS's list look more
                        ; complete.
                        lda curtask
                        asl
                        tax
                        lda #<id_ps
                        ldy #>id_ps
                        jsr setpridenty

                        ;
                        ldx #<prgtext1
                        ldy #>prgtext1
                        jsr puts

                        ;
                        lda #0
                        sta var1
                        ;
main_l1                 ldx var1
                        jsr getprstate
                        cmp #dead ; Do not list dead processes.
                        beq deadl

                        ;
                        ; Print process information on screen ..

                        ;
                        ; Print PID
                        lda var1
                        jsr phex4
                        lda #"."
                        jsr putch
                        lda #" "
                        jsr putch

                        ; Print Identy
                        lda #"'"
                        jsr putch
                        lda var1
                        asl
                        tax 
                        jsr getpridenty
                        cpx #0
                        bne loop1
                        cpy #0
                        bne loop1
                        lda #"?" ; Infotxt not available?
                        jsr putch
                        jmp loop2
loop1                   jsr puts ; Infotxt available?
loop2                   lda #"'"
                        jsr putch
                        lda #" "
                        jsr putch
                        
                        ; Then print process state
                        ;
                        lda #"'"
                        jsr putch
                        ;
                        ldx var1
                        jsr getprstate
                        and #3 ; Make it valid value
                        asl
                        tax
                        lda ptab,x
                        pha
                        lda ptab+1,x
                        tay
                        pla
                        tax
                        jsr puts
                        ;
                        lda #"'"
                        jsr putch

                        ;
                        lda #13
                        jsr putch                          

deadl                   nop
                        inc var1 ; Next process please ...
                        lda var1
                        cmp #$10 ; Until max. 16 processes listed
                        bne main_l1

                        ;
                        jsr releasestdright
                        jmp suicide

var1                    .byte 0

ptab                    .word s00,s01,s02,s03

s00                     .text "dead"
                        .byte 0

s01                     .text "worker"
                        .byte 0

s02                     .text "idler"
                        .byte 0

s03                     .text "?"
                        .byte 0



prgtext1                .text "ps 0.0.1"
                        .byte 13
                        .text "[pid] [des] [sta]"
                        .byte 13
                        .byte 0
