;
; MTASK's Include File for Applications v0.0.1
;

; KERNEL AT xxxx
kernel=$8000

;
version=$0100
stackl=$40 ; $00-$f0, size of the stack, lower=more, higher=less
           ; (Note: Stack grows backwards)
           ; With larger stacks it takes more time to switch
           ; between tasks, so think carefully do you really
           ; need huge $f0 byte f.e. stacks?
           ; $40 is actually quite nice value.
           ; Even on SCPU this really matters, even if
           ; task switching doesn't take time it anyway decreases
           ; the time that could be used more wisely.

; Memory region kernel-$1000 contains storage space for
; 16 processes stacks.
stack1=kernel-$1000  ; Virtual stack1 location in physical memory
stack2=kernel-$0f00  ; --------stack2--------
stack3=kernel-$0e00  ; --------stack3--------
stack4=kernel-$0d00  ; --------stack4--------
stack5=kernel-$0c00  ; --------stack5--------
stack6=kernel-$0b00  ; --------stack6--------
stack7=kernel-$0a00  ; --------stack7--------
stack8=kernel-$0900  ; --------stack8--------
stack9=kernel-$0800  ; --------stack9--------
stacka=kernel-$0700  ; --------stacka--------
stackb=kernel-$0600  ; --------stackb--------
stackc=kernel-$0500  ; --------stackc--------
stackd=kernel-$0400  ; --------stackd--------
stacke=kernel-$0300  ; --------stacke--------
stackf=kernel-$0200  ; --------stackf--------
stack10=kernel-$0100 ; --------stack10--------


; Task mode const's
dead=0
worker=1
idler=2
null=0

; REU I/O
reu_status   = $DF00
reu_command  = $DF01
reu_c64base  = $DF02
reu_reubase  = $DF04
reu_translen = $DF07
reu_irqmask  = $DF09
reu_control  = $DF0A

; SYSTEM VARIABLES (Critical!)
baswarm=$0302 ; Basic warm start address
r0=$3d0 ; Virtual 32bit registers
r1=$3d4
r2=$3d8
r3=$3dc
kflag=$028d ; KEYBOARD FLAG
stdin=$9e00 ; STDIN buffer
stderr=$9f00 ; STDERR - Errors buffer here
cpar=$03e0   ; C compiler support reserved!
power=$03f0  ; 32bit value that tells how much horse power
             ; MTASK has left to run new threads.
icallfreq=$3f4 ; Interrupt Calling Frequency
pristd=$03fb ; PRIMARY STD CASING :
             ; If this variable is set then
             ; all STD routines will be executed
             ; in non task switchable mode.
             ; (If multitasking is slowing down one single process,
             ;  then it is possible to set this variable to
             ;  make screen output look smoother atleast).
stdright=$03fc ; Don't set manually!(0=Rights not reserved,
               ; !=thread id that has reserved the std IN/OUT rights.)
curtask=$03fd ; Currently running task
action=$03fe
alts=$03ff ;Allow task switch (0=False, 1=True)
delayv=16 ; This value sets the frequence of task scheduling,
          ; smaller value = quicker
          ; larger value = slower
          ; Quicker means that more CPU power will be reserved for
          ; task switching.(Default 16 is just fine)
          ; Too small values(like 1-8) may jam the processor on
          ; C64's without SCPU or any other speeder.
maxtasks=15 ; Valid values = 1,3,7,15,...
            ; 1=2, 3=4, 7=8, 15=16, .. (acts like AND)


; Kernel Calls
install=        kernel+$00*3
engine=         kernel+$01*3
irq=            kernel+$02*3
load=           kernel+$03*3
save=           kernel+$04*3
calstrlen=      kernel+$05*3
printstr=       kernel+$06*3
clrscr=         kernel+$07*3
getch=          kernel+$08*3
putch=          kernel+$09*3
getprstate=     kernel+$0a*3
setprstate=     kernel+$0b*3
getprsleepie=   kernel+$0c*3
setprsleepie=   kernel+$0d*3
gets=           kernel+$0e*3
puts=           kernel+$0f*3
reservestdright=kernel+$10*3
releasestdright=kernel+$11*3
workmode=       kernel+$12*3
idlemode=       kernel+$13*3
suicide=        kernel+$14*3
spawn=          kernel+$15*3
kill=           kernel+$16*3
getworkers=     kernel+$17*3
getidlers=      kernel+$18*3
getdead=        kernel+$19*3
phex16=         kernel+$1a*3
phex4=          kernel+$1b*3
horsepower=     kernel+$1c*3
csup=           kernel+$1d*3
cntarg=         kernel+$1e*3
seekarg=        kernel+$1f*3
dset=           kernel+$20*3
dopen=          kernel+$21*3
dclose=         kernel+$22*3
dget=           kernel+$23*3
dput=           kernel+$24*3
dsetread=       kernel+$25*3
dsetwrite=      kernel+$26*3
dinit=          kernel+$27*3
calstrlen2=     kernel+$28*3
dname=          kernel+$29*3
waitnokey=      kernel+$2a*3
getpridenty=    kernel+$2b*3
setpridenty=    kernel+$2c*3
movereutoram=   kernel+$2d*3
moveramtoreu=   kernel+$2e*3

;       movereutoram( offsetreu offsetram length );
;       moveramtoreu( offsetreu offsetram length );
;
*=$2000
.text "mz"
.word main

main                    nop
                        ;
                        jsr reservestdright
                        ;
                        ldx #<prgtxt1
                        ldy #>prgtxt1
                        jsr puts

                        ; Set REU offset
                        lda #0
                        ldx #0
                        ldy #0
                        jsr setr0
                        ; Set RAM offset
                        lda #<tmp
                        ldx #>tmp
                        ldy #0
                        jsr setr1
                        ; Set length
                        lda #<256 
                        ldx #>256
                        ldy #0
                        jsr setr2

                        ; Count memory
                        lda #0
                        sta $30
                        sta $31
loop2                   nop
                        ;
                        ldx #0
loop1                   eor $dc04
                        sta tmp,x
                        sta tmp+256,x
                        inx
                        bne loop1
                        ;
                        jsr moveramtoreu
                        ; Make appropriate error on data.
                        ldx #0
                        txa
loop5                   lda #0
                        sta tmp,x
                        inx
                        bne loop5
                        jsr movereutoram
                        ldx #0
loop3                   lda tmp+256,x
                        cmp tmp,x
                        bne loop4 ; Memory false ? -> Max. linear address reached (0=means no REU at all)
                        inx
                        bne loop3
                        ; Increase block meter
                        jsr i30
                        ; Change page
                        inc r0+1
                        inc r0+1
                        inc r0+1
                        inc r0+1
                        bne br0
                        ldx r0+1
                        lda r0+2
                        jsr phex16
                        lda #13
                        jsr putch
                        inc r0+2
                        bne br0
                        ; Are we overflowing 16M limit? Yeah->Show result.
                        jmp loop4
br0                     nop
                        jmp loop2
loop4                   nop
                        ;
                        ldx $30
                        lda $31
                        jsr phex16
                        ;
                        ldx #<prgtxt2
                        ldy #>prgtxt2
                        jsr puts
                        ;
                        jsr releasestdright
                        jmp suicide

prgtxt1                 .text "reu memory tester version 0.0.1"
                        .byte 13
                        .byte 0

prgtxt2                 .text " KB(s) of reu memory detected."
                        .byte 13
                        .byte 0

setr0                   sta r0
                        stx r0+1
                        sty r0+2
                        rts

setr1                   sta r1
                        stx r1+1
                        sty r1+2
                        rts

setr2                   sta r2
                        stx r2+1
                        sty r2+2
                        rts

setr3                   sta r3
                        stx r3+1
                        sty r3+2
                        rts

i30                     inc $30
                        bne b30
                        inc $31
b30                     rts

; Keep this var right here !
tmp                     .byte 0

